/*----------------------------------------------------------------------------* 
**  "comp5_3.cpp"  using the Runge-Kutta numerical differentiation method
**  for solving differential equations.
**
**  To build this in Visual C++ 6.0 perform the following steps:
**
**  1. Select New from File menu.
**
**  2. In the New dialog box select "Win32 Application" and enter 
**     a project name.  Then press the "Ok" button.
**
**  3. In the Win32 Application dialog box select "A simple Win32 Application" 
**     then press the "Finish" button.  
**
**  4. Press "Ok" in the New Project Information dialog box.
**
**  5. Open the *.cpp file for entry of the program.  (We do not edit StdAfx.h
**     or StdAfx.cpp; VC++ requires these files.)
*/

#include "stdafx.h"
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <windowsx.h>
#include "..\plotter\plotter.h"
#include "..\vector\vector.h"

plotter np( "Plotter", "Numerical Plotter" ); 

/*----------------------------------------------------------------------------*
**  "WinMain" is the program entry point
**
**  returns: message parameter
*/
int APIENTRY WinMain( HINSTANCE inst,           // window instance
                      HINSTANCE prev,           // previous window
                      LPSTR     cmdl,           // command line string
                      int       show )          // initial display
{
    np.initialize( inst );
    
    if( !np.window_class() )
	return 0;
  
    np.create( show );
 
    return np.message_loop();
}
    
/*----------------------------------------------------------------------------*
**  "WindowFunc" processes messages to this application
**
**  returns: 0 if message processed, else default value
*/
LRESULT CALLBACK WindowFunc( HWND   hwnd,       // handle to window of message
			     UINT   msg,        // message code
			     WPARAM wp,         // short message amplifier
			     LPARAM lp )        // long message amplifier
{
    switch( msg )
    {
    case WM_CREATE:
	np.on_create( hwnd );
	break;

    case WM_DESTROY:
	np.on_destroy();
	break;

    case WM_SIZE:
	np.on_size( hwnd );
	break;

    case WM_HSCROLL:
	np.on_hscroll( hwnd, wp );
	break;

    case WM_VSCROLL:
	np.on_vscroll( hwnd, wp );
	break;

    case WM_PAINT:
	np.on_paint( hwnd );
	break;

    case WM_COMMAND:
	np.on_command( hwnd, wp );
	break;

    default:
	return DefWindowProc( hwnd, msg, wp, lp );
    }

    return 0;
}

/*----------------------------------------------------------------------------*\
**              Put your mathematical functions to plot here.                 **
\*----------------------------------------------------------------------------*/

int calculate( POINT *, POINT *, int );
double rk( double, double, double, int, double (g)(double, double) );
double G( double, double );
double U( double );

/*----------------------------------------------------------------------------*
**  "run" performs calculations and plots results.  
**
**  This is a shell in which the function calls for a specific calculation
**  are entered.
**
**  returns: nothing
*/
void run()
{
    POINT curve1[100], curve2[100];
    int i;
            
    // first, define a plot region 
    // this must be done before calculating points to be plotted
    np.plot_region( -0.001, 0.01, 2, -1 );

    // do your calculations
    i = calculate( curve1, curve2, 100 );

    // plot the results
    np.draw_axes();   
    np.green_curve( curve1, i );
    np.blue_curve( curve2, i );
}

/*----------------------------------------------------------------------------*
**  "calculate" is a shell for the calculation algorithms.
**
**  returns: number of items in POINT array
*/
int calculate( POINT *a0, POINT *a1, int points )
{
    int i;
    double y       = 0.0; 
    double t       = 0.0;
    double delta   = 0.0001;
    double y_scale = 1000.0;

    a0[0] = np.plot_point( t, y * y_scale );
    a1[0] = np.plot_point( t, U( t ) );

    for( i = 1; i < points; i++ )
    {
	y = rk( t, y, delta, 10, G );
	t = t + delta;
	
        a0[i] = np.plot_point( t, y * y_scale );
        a1[i] = np.plot_point( t, U( t ) );
    }
    return i;
}

/*----------------------------------------------------------------------------* 
**  "rk" performs numerical differentiation using the 4-th order 
**  Runge Kutta algorithm to solve differential equations of the form: 
**
**                      y' = g( t, y )
**
**  returns: the value of y at end of interval
*/
double rk( double t_beg,                        // interval start
	   double y_beg,                        // y at start
	   double delta,                        // interval duration
	   int    n,                            // number of segments > 0 
	   double (g)(double, double) )         // integrand function
{
    double t = t_beg;                           // initialize time
    double y = y_beg;                           // initialize variable
    double dt = delta / n;                      // initialize step duration

    double g1, g2, g3, g4;                      // variables for computation
    double dt2 = dt / 2.0;
    double dt6 = dt / 6.0;
    int    i;                                   // loop counter

    for( i = 0; i < n; i++ )                    // do n steps of R-K algorithm
    {
	g1 = g( t,       y );
	g2 = g( t + dt2, y + g1 * dt2 );
	g3 = g( t + dt2, y + g2 * dt2 );
	g4 = g( t + dt,  y + g3 * dt );

	y += ( g1 + g2 * 2.0 + g3 * 2.0 + g4 ) * dt6;
	t += dt;
    }

    return y;
}

/*----------------------------------------------------------------------------* 
**  "U" implements an input function of the form: 
**
**                      u(t)
**
**  returns: the instantaneous value of u corresponding to time t
*/
double U( double t )
{ 
    return ( t <= 0.003 ) ? 1.0 : 0.0;
}

/*----------------------------------------------------------------------------* 
**  "G" implements a function of the form: 
**
**                      y' = g( t, y ) = G(t) y + u(t)
**
**  returns: the instantaneous value of y' corresponding to time t
*/
double G( double t, double y )
{
    double yp;

    yp =  - 1000.0 * y + U( t );

    return yp;
}


